@@ -2,6 +2,7 @@
 
 #include <QScopedPointer>
 #include <QtNetwork>
+#include <QJsonDocument>
 
 #include "account.h"
 
@@ -50,6 +51,8 @@ const char* kRemoteWipeReportUrl = "api2/device-wiped/";
 const char* kSearchUsersUrl = "api2/search-user/";
 const char* kGetThumbnailUrl = "api2/repos/%1/thumbnail/";
 const char* kCreateFileUploadLink = "api/v2.1/upload-links/";
+const char* kClientSSOLinkUrl = "api2/client-sso-link/";
+const char* kClientSSOStatusUrl = "api2/client-sso-link/%1/";
 
 const char* kGetFileActivitiesUrl = "api/v2.1/activities/";
 } // namespace
@@ -150,13 +153,15 @@ DownloadRepoRequest::DownloadRepoRequest(const Account& account,
           account.getAbsoluteUrl("api2/repos/" + repo_id + "/download-info/"),
           SeafileApiRequest::METHOD_GET,
           account.token),
-      read_only_(read_only)
+      read_only_(read_only),
+      account_(account)
 {
 }
 
 RepoDownloadInfo RepoDownloadInfo::fromDict(QMap<QString, QVariant>& dict,
                                             const QUrl& url_in,
-                                            bool read_only)
+                                            bool read_only,
+                                            const QString& username)
 {
     RepoDownloadInfo info;
     info.repo_version = dict["repo_version"].toInt();
@@ -176,8 +181,9 @@ RepoDownloadInfo RepoDownloadInfo::fromDict(QMap<QString, QVariant>& dict,
     QString salt = dict.value("salt").toString();
     QMap<QString, QVariant> map;
     map.insert("is_readonly", read_only ? 1 : 0);
-    map.insert("server_url", url.toString());
+    map.insert("server_url", url.toString(QUrl::FullyEncoded));
     map.insert("repo_salt", salt);
+    map.insert("username", username);
 
     info.more_info = ::mapToJson(map);
 
@@ -197,7 +203,7 @@ void DownloadRepoRequest::requestSuccess(QNetworkReply& reply)
     QScopedPointer<json_t, JsonPointerCustomDeleter> json(root);
     QMap<QString, QVariant> dict = mapFromJSON(json.data(), &error);
 
-    RepoDownloadInfo info = RepoDownloadInfo::fromDict(dict, url(), read_only_);
+    RepoDownloadInfo info = RepoDownloadInfo::fromDict(dict, url(), read_only_, account_.accountInfo.name);
 
     emit success(info);
 }
@@ -240,7 +246,9 @@ CreateRepoRequest::CreateRepoRequest(const Account& account,
                                      const QString& passwd)
     : SeafileApiRequest(account.getAbsoluteUrl(kCreateRepoUrl),
                         SeafileApiRequest::METHOD_POST,
-                        account.token)
+                        account.token),
+      account_(account)
+
 {
     setFormParam(QString("name"), name);
     setFormParam(QString("desc"), desc);
@@ -302,7 +310,7 @@ void CreateRepoRequest::requestSuccess(QNetworkReply& reply)
 
     QScopedPointer<json_t, JsonPointerCustomDeleter> json(root);
     QMap<QString, QVariant> dict = mapFromJSON(json.data(), &error);
-    RepoDownloadInfo info = RepoDownloadInfo::fromDict(dict, url(), false);
+    RepoDownloadInfo info = RepoDownloadInfo::fromDict(dict, url(), false, account_.accountInfo.name);
 
     emit success(info);
 }
@@ -715,6 +723,12 @@ ServerInfoRequest::ServerInfoRequest(const Account& account)
 {
 }
 
+ServerInfoRequest::ServerInfoRequest(const QUrl& server_url)
+    : SeafileApiRequest(urlJoin(server_url, kServerInfoUrl),
+                        SeafileApiRequest::METHOD_GET)
+{
+}
+
 void ServerInfoRequest::requestSuccess(QNetworkReply& reply)
 {
     json_error_t error;
@@ -750,7 +764,7 @@ void ServerInfoRequest::requestSuccess(QNetworkReply& reply)
         ret.customBrand = dict["desktop-custom-brand"].toString();
     }
 
-    emit success(account_, ret);
+    emit success(ret);
 }
 
 LogoutDeviceRequest::LogoutDeviceRequest(const Account& account)
@@ -1429,3 +1443,64 @@ void GetUploadFileLinkRequest::requestSuccess(QNetworkReply& reply)
     } while (0);
     emit failed(ApiError::fromHttpError(500));
 }
+
+ClientSSOLinkRequest::ClientSSOLinkRequest(const QUrl& server_url)
+    : SeafileApiRequest(urlJoin(server_url, kClientSSOLinkUrl),
+                        SeafileApiRequest::METHOD_POST)
+{
+}
+
+void ClientSSOLinkRequest::requestSuccess(QNetworkReply& reply)
+{
+    json_error_t error;
+    json_t *root = parseJSON(reply, &error);
+    if (!root) {
+        qWarning() << "ClientSSOLinkRequest: failed to parse json:" << error.text;
+        emit failed(ApiError::fromJsonError());
+        return;
+    }
+
+    QScopedPointer<json_t, JsonPointerCustomDeleter> json(root);
+    QMap<QString, QVariant> dict = mapFromJSON(json.data(), &error);
+    QString link = dict["link"].toString();
+    if (link.isEmpty()) {
+        qWarning() << "ClientSSOLinkRequest: failed to parse link";
+        emit failed(ApiError::fromJsonError());
+        return;
+    }
+
+    emit success(link);
+}
+
+ClientSSOStatusRequest::ClientSSOStatusRequest(const QUrl& server_url, const QString& token)
+    : SeafileApiRequest(urlJoin(server_url, QString(kClientSSOStatusUrl).arg(token)),
+                        SeafileApiRequest::METHOD_GET)
+{
+}
+
+void ClientSSOStatusRequest::requestSuccess(QNetworkReply& reply)
+{
+    json_error_t error;
+    json_t *root = parseJSON(reply, &error);
+    if (!root) {
+        qWarning() << "ClientSSOStatusRequest: failed to parse json:" << error.text;
+        emit failed(ApiError::fromJsonError());
+        return;
+    }
+
+    QScopedPointer<json_t, JsonPointerCustomDeleter> json(root);
+    QMap<QString, QVariant> dict = mapFromJSON(json.data(), &error);
+
+    ClientSSOStatus status;
+    status.status = dict["status"].toString();
+    status.username = dict["username"].toString();
+    status.api_token = dict["apiToken"].toString();
+
+    if (status.status.isEmpty()) {
+        qWarning() << "ClientSSOStatusRequest: failed to parse status";
+        emit failed(ApiError::fromJsonError());
+        return;
+    }
+
+    emit success(status);
+}
