@@ -12,6 +12,7 @@
 #include "api/server-repo.h"
 #include "api/starred-file.h"
 #include "api/event.h"
+#include "api/sso-status.h"
 
 class QNetworkReply;
 class QImage;
@@ -90,7 +91,8 @@ public:
 
     static RepoDownloadInfo fromDict(QMap<QString, QVariant>& dict,
                                      const QUrl& url,
-                                     bool read_only);
+                                     bool read_only,
+                                     const QString& username);
 };
 
 class DownloadRepoRequest : public SeafileApiRequest
@@ -112,6 +114,7 @@ private:
     Q_DISABLE_COPY(DownloadRepoRequest)
 
     bool read_only_;
+    Account account_;
 };
 
 class GetRepoRequest : public SeafileApiRequest
@@ -170,6 +173,7 @@ signals:
 
 private:
     Q_DISABLE_COPY(CreateRepoRequest)
+    Account account_;
 };
 
 class CreateSubrepoRequest : public SeafileApiRequest
@@ -406,9 +410,15 @@ class ServerInfoRequest : public SeafileApiRequest
     Q_OBJECT
 public:
     ServerInfoRequest(const Account& account);
+    ServerInfoRequest(const QUrl& server_url);
+
+    const Account& account() const
+    {
+        return account_;
+    }
 
 signals:
-    void success(const Account& account, const ServerInfo& info);
+    void success(const ServerInfo& info);
 
 protected slots:
     void requestSuccess(QNetworkReply& reply);
@@ -887,4 +897,38 @@ private:
     Q_DISABLE_COPY(GetUploadFileLinkRequest);
 };
 
+class ClientSSOLinkRequest : public SeafileApiRequest
+{
+    Q_OBJECT
+
+public:
+    ClientSSOLinkRequest(const QUrl& server_url);
+
+signals:
+    void success(const QString& link);
+
+protected slots:
+    void requestSuccess(QNetworkReply& reply);
+
+private:
+    Q_DISABLE_COPY(ClientSSOLinkRequest)
+};
+
+class ClientSSOStatusRequest : public SeafileApiRequest
+{
+    Q_OBJECT
+
+public:
+    ClientSSOStatusRequest(const QUrl& server_url, const QString& token);
+
+signals:
+    void success(const ClientSSOStatus& status);
+
+protected slots:
+    void requestSuccess(QNetworkReply& reply);
+
+private:
+    Q_DISABLE_COPY(ClientSSOStatusRequest)
+};
+
 #endif // SEAFILE_CLIENT_API_REQUESTS_H
