@@ -351,7 +351,7 @@ void FileBrowserDialog::createToolBar()
     toolbar_->widgetForAction(gohome_action_)->setObjectName("goHomeButton");
 
     path_navigator_ = new QButtonGroup(this);
-    connect(path_navigator_, SIGNAL(buttonClicked(int)),
+    connect(path_navigator_, SIGNAL(idClicked(int)),
             this, SLOT(onNavigatorClick(int)));
 
     search_toolbar_ = new QToolBar;
@@ -556,6 +556,22 @@ void FileBrowserDialog::updateFileCount()
     details_label_->setText(tr("%1 items").arg(row_count));
 }
 
+void FileBrowserDialog::setUnlockedFilesWritable(const QList<SeafDirent>& dirents)
+{
+    for (auto&& file : dirents) {
+        QString fpath = ::pathJoin(current_path_, file.name);
+        QString cached_file = data_mgr_->getLocalCacheFilePath(repo_.id, fpath);
+        if (!cached_file.isEmpty() && QFileInfo(cached_file).exists() && !file.is_locked) {
+            if (!(QFile::permissions(cached_file) & QFileDevice::WriteOwner)) {
+                QFile::setPermissions(cached_file,
+                    QFileDevice::ReadOwner | QFileDevice::WriteOwner |
+                    QFileDevice::ReadGroup | QFileDevice::ReadOther
+                );
+            }
+        }
+    }
+}
+
 void FileBrowserDialog::fetchDirents(bool force_refresh)
 {
     if (!has_password_dialog_ && repo_.encrypted && !data_mgr_->isRepoPasswordSet(repo_.id)) {
@@ -798,6 +814,7 @@ void FileBrowserDialog::enterPath(const QString& path)
 void FileBrowserDialog::onFileClicked(const SeafDirent& file)
 {
     QString fpath = ::pathJoin(current_path_, file.name);
+    qInfo() << "[FileBrowser] double clicked file" << fpath;
 
     LocalRepo repo;
     if (seafApplet->rpcClient()->getLocalRepo(repo_.id, &repo) == 0) {
@@ -809,20 +826,46 @@ void FileBrowserDialog::onFileClicked(const SeafDirent& file)
         openFile(synced_path);
         return;
     }
+
     QString cached_file = data_mgr_->getLocalCachedFile(repo_.id, fpath, file.id);
-    qDebug("cached_file is %s", cached_file.toUtf8().data());
     if (!cached_file.isEmpty() && QFileInfo(cached_file).exists()) {
+        QFileInfo info(cached_file);
+        qInfo() << "[FileBrowser] open cached file" << fpath
+                << ", size =" << info.size() << ", mtime =" << info.lastModified();
+        // Cached files are set to read-only when they are locked.
+        if (file.is_locked) {
+            QFile::setPermissions(cached_file,
+                QFileDevice::ReadOwner |
+                QFileDevice::ReadGroup |
+                QFileDevice::ReadOther
+            );
+        }
+
         // double-checked the watch, since it might fails sometime
         AutoUpdateManager::instance()->watchCachedFile(account_, repo_.id, fpath);
         openFile(cached_file);
         return;
     } else {
         if (TransferManager::instance()->getDownloadTask(repo_.id, fpath)) {
+            qInfo() << "[FileBrowser] file is downloading" << fpath;
             return;
         }
-        AutoUpdateManager::instance()->removeWatch(
-            DataManager::getLocalCacheFilePath(repo_.id, fpath));
-        downloadFile(fpath);
+        qInfo() << "[FileBrowser] file will be downloaded" << fpath;
+
+        auto local_file = DataManager::getLocalCacheFilePath(repo_.id, fpath);
+        AutoUpdateManager::instance()->removeWatch(local_file);
+
+        // The read-only permission will be cleared before downloading files to avoid updating errors.
+        if (!local_file.isEmpty() && QFileInfo(local_file).exists()) {
+            if (!(QFile::permissions(local_file) & QFileDevice::WriteOwner)) {
+                QFile::setPermissions(local_file,
+                    QFileDevice::ReadOwner | QFileDevice::WriteOwner |
+                    QFileDevice::ReadGroup | QFileDevice::ReadOther
+                );
+            }
+        }
+
+        downloadFile(fpath, file.is_locked);
     }
 }
 
@@ -831,11 +874,12 @@ void FileBrowserDialog::createDirectory(const QString &name)
     data_mgr_->createDirectory(repo_.id, ::pathJoin(current_path_, name));
 }
 
-void FileBrowserDialog::downloadFile(const QString& path)
+void FileBrowserDialog::downloadFile(const QString& path, bool is_locked)
 {
-    qDebug("begin to downloadfile is %s", path.toUtf8().data());
     FileDownloadTask *task = data_mgr_->createDownloadTask(repo_.id, path);
-    connect(task, SIGNAL(finished(bool)), this, SLOT(onDownloadFinished(bool)));
+    connect(task, &FileDownloadTask::finished, this, [=](bool success) {
+        onDownloadFinished(success, is_locked);
+    });
 }
 
 void FileBrowserDialog::onGetDirentReupload(const SeafDirent& dirent)
@@ -848,8 +892,10 @@ void FileBrowserDialog::onGetDirentReupload(const SeafDirent& dirent)
 void FileBrowserDialog::uploadFile(const QString& path, const QString& name,
                                    bool overwrite)
 {
-    FileUploadTask *task =
-      data_mgr_->createUploadTask(repo_.id, current_path_, path, name, overwrite);
+    FileCache::CacheEntry entry;
+    FileCache::instance()->getCacheEntry(repo_.id, path, &entry);
+
+    FileUploadTask *task = data_mgr_->createUploadTask(repo_.id, current_path_, path, entry.commit_id, name, overwrite);
     connect(task, SIGNAL(finished(bool)), this, SLOT(onUploadFinished(bool)));
     FileBrowserProgressDialog *dialog = new FileBrowserProgressDialog(task, this);
     task->start();
@@ -918,6 +964,7 @@ void FileBrowserDialog::uploadFileOrMkdir()
 
 void FileBrowserDialog::uploadOrUpdateFile(const QString& path)
 {
+    qInfo() << "[FileBrowser] upload or update file:" << path;
     const QString name = ::getBaseName(path);
 
     // ignore the confirm procedure for uploading directory, which is non-sense
@@ -948,24 +995,38 @@ void FileBrowserDialog::uploadOrUpdateFile(const QString& path)
 
 void FileBrowserDialog::uploadOrUpdateMutipleFile(const QStringList &paths)
 {
+    qInfo() << "[FileBrowser] upload or update files:" << paths;
     if (paths.size() == 1)
         uploadOrUpdateFile(paths.front());
     else
         uploadMultipleFile(paths);
 }
 
-void FileBrowserDialog::onDownloadFinished(bool success)
+void FileBrowserDialog::onDownloadFinished(bool success, bool is_file_locked)
 {
     FileDownloadTask *task = qobject_cast<FileDownloadTask *>(sender());
     QString _error;
     if (task == NULL)
         return;
     if (success) {
-        if (!task->isSaveAsTask())
+        if (!task->isSaveAsTask()) {
+            QFileInfo info(task->localFilePath());
+            qInfo() << "[FileBrowser] download file success" << task->path()
+                    << ", size =" << info.size() << ", mtime =" << info.lastModified();
+
+            // Cached files are set to read-only when they are locked.
+            if (is_file_locked) {
+                QFile::setPermissions(task->localFilePath(),
+                    QFileDevice::ReadOwner |
+                    QFileDevice::ReadGroup |
+                    QFileDevice::ReadOther
+                );
+            }
             openFile(task->localFilePath());
+        }
     } else {
         if (repo_.encrypted &&
-            setPasswordAndRetry(task)) {
+            setPasswordAndRetry(task, is_file_locked)) {
             return;
         }
 
@@ -1014,6 +1075,10 @@ void FileBrowserDialog::onUploadFinished(bool success)
             _error = tr("Library/Folder not found.");
         } else if (task->httpErrorCode() == 401) {
             _error = tr("Authorization expired");
+        } else if (task->httpErrorCode() == 442) {
+            _error = tr("File size exceeds limit");
+        } else if (task->httpErrorCode() == 447) {
+            _error = tr("Number of file exceeds limit");
         } else {
             _error = task->errorString();
         }
@@ -1075,7 +1140,7 @@ void FileBrowserDialog::onUploadFinished(bool success)
     updateFileCount();
 }
 
-bool FileBrowserDialog::setPasswordAndRetry(FileNetworkTask *task)
+bool FileBrowserDialog::setPasswordAndRetry(FileNetworkTask *task, bool is_file_locked)
 {
     if (task->httpErrorCode() == 400) {
         if (has_password_dialog_) {
@@ -1085,7 +1150,7 @@ bool FileBrowserDialog::setPasswordAndRetry(FileNetworkTask *task)
         SetRepoPasswordDialog password_dialog(repo_, this);
         if (password_dialog.exec() == QDialog::Accepted) {
             if (task->type() == FileNetworkTask::Download)
-                downloadFile(task->path());
+                downloadFile(task->path(), is_file_locked);
             else
                 uploadOrUpdateFile(task->path());
             return true;
@@ -1150,6 +1215,10 @@ void FileBrowserDialog::updateTable(const QList<SeafDirent>& dirents)
     }
     gohome_action_->setEnabled(current_path_ != "/");
     updateFileCount();
+
+    // The read-only permission will be cleared on unlocked files when refreshing.
+    // However, we will not assign the read-only permission on locked files, because it is done when opening files. Also, other measures has been taken to prevent uploading of diverged files.
+    setUnlockedFilesWritable(dirents);
 }
 
 void FileBrowserDialog::chooseFileToUpload()
@@ -1425,7 +1494,7 @@ void FileBrowserDialog::onDirentShareFailed(const ApiError&error)
 
 void FileBrowserDialog::onFileAutoUpdated(const QString& repo_id, const QString& path)
 {
-    if (repo_id == repo_.id && path == current_path_) {
+    if (repo_id == repo_.id) {
         forceRefresh();
     }
 }
@@ -1529,7 +1598,7 @@ void FileBrowserDialog::onGetSyncSubdirectory(const QString &folder_name)
 void FileBrowserDialog::onDeleteLocalVersion(const SeafDirent &dirent)
 {
     QString fpath = ::pathJoin(current_path_, dirent.name);
-    QString cached_file = data_mgr_->getLocalCachedFile(repo_.id, fpath, dirent.id);
+    QString cached_file = data_mgr_->getLocalCacheFilePath(repo_.id, fpath);
     if (!cached_file.isEmpty() && QFileInfo(cached_file).exists()) {
         QFile::remove(cached_file);
         return;
@@ -1543,7 +1612,7 @@ void FileBrowserDialog::onLocalVersionSaveAs(const SeafDirent &dirent)
        download_dir = QDir::home();
 
      QString fpath = ::pathJoin(current_path_, dirent.name);
-     QString cached_file = data_mgr_->getLocalCachedFile(repo_.id, fpath, dirent.id);
+     QString cached_file = data_mgr_->getLocalCacheFilePath(repo_.id, fpath);
      if (!cached_file.isEmpty() && QFileInfo(cached_file).exists()) {
          QString local_path = QFileDialog::getSaveFileName(this, tr("Enter name of file to save to..."), download_dir.filePath(dirent.name));
          QFile::copy(cached_file, local_path);
