@@ -6,6 +6,7 @@
 #include <QTimer>
 
 #include "utils/utils.h"
+#include "ui/repo-tree-model.h"
 #include "utils/utils-mac.h"
 #include "seafile-applet.h"
 #include "ui/tray-icon.h"
@@ -56,10 +57,10 @@ const char *kProxyPort = "proxy_port";
 const char *kProxyUsername = "proxy_username";
 const char *kProxyPassword = "proxy_password";
 const char *kHideWindowsIncompatiblePathNotification = "hide_windows_incompatible_path_notification";
+const char *kRepoSortOrdersGroup = "repo_sort_orders";
 
 const int kCheckSystemProxyIntervalMSecs = 5 * 1000;
 
-
 #ifdef Q_OS_WIN32
 QString softwareSeafile()
 {
@@ -163,6 +164,9 @@ void SettingsManager::loadSettings()
 
     autoStart_ = get_seafile_auto_start();
 
+    QSettings settings;
+    repo_sort_orders_ = settings.value(kRepoSortOrdersGroup).toInt();
+
 #ifdef HAVE_FINDER_SYNC_SUPPORT
     // try to do a reinstall, or we may use findersync somewhere else
     // this action won't stop findersync if running already
@@ -187,7 +191,13 @@ void SettingsManager::loadProxySettings()
 
     QString use_proxy;
     seafApplet->rpcClient()->seafileGetConfig(kUseProxy, &use_proxy);
-    if (use_proxy != "true") {
+    if (use_proxy.isNull()) {
+        // The use_proxy is null when no proxy setting has been configured.
+        // So we use the "system proxy" setting as default and notify daemon.
+        current_proxy_.type = SystemProxy;
+        writeProxySettingsToDaemon(current_proxy_);
+        return;
+    } else if (use_proxy != "true") {
         return;
     }
     QString use_system_proxy;
@@ -231,9 +241,19 @@ void SettingsManager::loadProxySettings()
         proxy.password = proxy_password;
 
     } else if (proxy_type == "socks") {
+        if (seafApplet->rpcClient()->seafileGetConfig(kProxyUsername,
+                                                      &proxy_username) < 0) {
+            return;
+        }
+        if (seafApplet->rpcClient()->seafileGetConfig(kProxyPassword,
+                                                      &proxy_password) < 0) {
+            return;
+        }
         proxy.type = SocksProxy;
         proxy.host = proxy_host;
         proxy.port = proxy_port;
+        proxy.username = proxy_username;
+        proxy.password = proxy_password;
     } else if (!proxy_type.isEmpty()) {
         qWarning("Unsupported proxy_type %s", proxy_type.toUtf8().data());
         return;
@@ -431,7 +451,7 @@ void SettingsManager::SeafileProxy::toQtNetworkProxy(QNetworkProxy *proxy) const
                                      : QNetworkProxy::Socks5Proxy);
     proxy->setHostName(host);
     proxy->setPort(port);
-    if (type == HttpProxy && !username.isEmpty() && !password.isEmpty()) {
+    if ((type == HttpProxy || type == SocksProxy) && !username.isEmpty() && !password.isEmpty()) {
         proxy->setUser(username);
         proxy->setPassword(password);
     }
@@ -456,6 +476,8 @@ SettingsManager::SeafileProxy SettingsManager::SeafileProxy::fromQtNetworkProxy(
         sproxy.password = proxy.password();
     } else if (proxy.type() == QNetworkProxy::Socks5Proxy) {
         sproxy.type = SocksProxy;
+        sproxy.username = proxy.user();
+        sproxy.password = proxy.password();
     }
 
     return sproxy;
@@ -473,7 +495,8 @@ bool SettingsManager::SeafileProxy::operator==(const SeafileProxy &rhs) const
                username == rhs.username && password == rhs.password;
     } else {
         // socks proxy
-        return host == rhs.host && port == rhs.port;
+        return host == rhs.host && port == rhs.port &&
+               username == rhs.username && password == rhs.password;
     }
 }
 
@@ -539,7 +562,7 @@ void SettingsManager::writeProxyDetailsToDaemon(const SeafileProxy& proxy)
     rpc->seafileSetConfig(kProxyType, type);
     rpc->seafileSetConfig(kProxyAddr, proxy.host.toUtf8().data());
     rpc->seafileSetConfigInt(kProxyPort, proxy.port);
-    if (type == "http") {
+    if (type == "http" || type == "socks") {
         rpc->seafileSetConfig(kProxyUsername, proxy.username.toUtf8().data());
         rpc->seafileSetConfig(kProxyPassword, proxy.password.toUtf8().data());
     }
@@ -615,6 +638,19 @@ void SettingsManager::setComputerName(const QString &computerName)
     seafApplet->rpcClient()->seafileSetConfig("client_name", computerName);
 }
 
+void SettingsManager::setRepoSortOrder(int order)
+{
+    QSettings settings;
+    settings.setValue(kRepoSortOrdersGroup, order);
+
+    repo_sort_orders_ = order;
+}
+
+int SettingsManager::repoSortOrder() const
+{
+    return repo_sort_orders_;
+}
+
 #ifdef HAVE_SHIBBOLETH_SUPPORT
 QString SettingsManager::getLastShibUrl()
 {
