@@ -57,11 +57,6 @@ const int kMessageDisplayTimeMSecs = 5000;
 const QString kShellExtFixExecutableName = "shellext-fix.exe";
 const QString kShellFixLogName = "shellext-fix.log";
 #endif
-#ifdef Q_OS_MAC
-void darkmodeWatcher(bool /*new Value*/) {
-    seafApplet->trayIcon()->reloadTrayIcon();
-}
-#endif
 
 QString folderToShow(const CommitDetails& details, const QString& worktree)
 {
@@ -154,9 +149,6 @@ void SeafileTrayIcon::start()
 {
     show();
     refresh_timer_->start(kRefreshInterval);
-#if defined(Q_OS_MAC)
-    utils::mac::set_darkmode_watcher(&darkmodeWatcher);
-#endif
     sync_errors_dialog_ = new SyncErrorsDialog;
     sync_errors_dialog_->updateErrors();
 }
@@ -319,7 +311,6 @@ void SeafileTrayIcon::showMessage(const QString &title,
                                   const QString &commit_id,
                                   const QString &previous_commit_id,
                                   MessageIcon icon,
-                                  int millisecondsTimeoutHint,
                                   bool is_error_message)
 {
     is_error_message_ = is_error_message;
@@ -337,7 +328,7 @@ void SeafileTrayIcon::showMessage(const QString &title,
     }
     // qWarning("using new style notifications");
 
-    QSystemTrayIcon::showMessage(title, message, icon, millisecondsTimeoutHint);
+    QSystemTrayIcon::showMessage(title, message, icon, 0);
 #elif defined(Q_OS_LINUX)
     repo_id_ = repo_id;
     Q_UNUSED(icon);
@@ -410,6 +401,10 @@ QIcon SeafileTrayIcon::getIcon(const QString& name)
     }
 
     QIcon icon(name);
+#ifdef Q_OS_MAC
+    // The icon style has been changed to monochrome on macOS.
+    icon.setIsMask(true);
+#endif
     icon_cache_[name] = icon;
     return icon;
 }
@@ -444,8 +439,6 @@ QIcon SeafileTrayIcon::stateToIcon(TrayState state)
     }
     return getIcon(icon_name);
 #elif defined(Q_OS_MAC)
-    bool isDarkMode = utils::mac::is_darkmode();
-    // filename = icon_name + ?white + .png
     QString icon_name;
 
     switch (state) {
@@ -453,7 +446,7 @@ QIcon SeafileTrayIcon::stateToIcon(TrayState state)
         icon_name = ":/images/mac/daemon_up";
         break;
     case STATE_DAEMON_DOWN:
-        icon_name = ":/images/mac/daemon_down.png";
+        icon_name = ":/images/mac/daemon_down";
         break;
     case STATE_DAEMON_AUTOSYNC_DISABLED:
         icon_name = ":/images/mac/seafile_auto_sync_disabled";
@@ -471,7 +464,7 @@ QIcon SeafileTrayIcon::stateToIcon(TrayState state)
         icon_name = ":/images/mac/notification";
         break;
     }
-    return getIcon(icon_name + (isDarkMode ? "_white" : "") + ".png");
+    return getIcon(icon_name + ".png");
 #else
     QString icon_name;
     switch (state) {
@@ -533,9 +526,8 @@ void SeafileTrayIcon::openHelp()
     if (QLocale::system().name() == "zh_CN") {
         url = "https://cloud.seafile.com/published/seafile-user-manual/syncing_client/install_syncing_client.md";
     } else {
-        url = "https://download.seafile.com/published/seafile-user-manual/syncing_client/install_sync.md";
+        url = "https://help.seafile.com/syncing_client/install_sync/";
     }
-
     QDesktopServices::openUrl(QUrl(url));
 }
 
@@ -741,5 +733,4 @@ void SeafileTrayIcon::showSyncErrorsDialog()
     sync_errors_dialog_->show();
     sync_errors_dialog_->raise();
     sync_errors_dialog_->activateWindow();
-    setSyncErrorStatus(false);
 }
